;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: cmx_dio_chan.asm
;;   Version: 1.0.0
;;
;;  DESCRIPTION:
;;    Digital Input Output algorithms.
;;
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress MicroSystems 2004. All Rights Reserved.
;;*****************************************************************************
;;  HISTORY:
;;
;;*****************************************************************************
include "m8c.inc"
include "memory.inc"
include "cmx_dio_chan.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------
export  _GlobalSelectConnect
export   GlobalSelectConnect

export  _GlobalSelectDisconnect
export   GlobalSelectDisconnect

export  _DiscreteDriveMode
export   DiscreteDriveMode

export  _SetPin
export   SetPin

export  _DISetPin
export   DISetPin

export  _DiscreteReadPin
export   DiscreteReadPin

export  _pinlu
export   pinlu

export  _portlu
export   portlu

export  _DIOChannelPins
export   DIOChannelPins

;-----------------------------------------------
;  Function Prototypes
;-----------------------------------------------

;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------
area text(ROM,REL)
.literal
_pinlu:
 pinlu:  db 01h, 02h, 04h, 08h, 10h, 20h, 40h, 80h
_portlu:
 portlu: db 00h, 04h, 08h, 0Ch, 10h, 14h, 18h, 1Ch

_DIOChannelPins:
 DIOChannelPins:
    db 00h
    db 02h
    db 03h
    db 04h
    db 05h
    db 06h
    db 20h
    db 21h
    db 22h
    db 23h
    db 24h
    db 25h
    db 26h
    db 27h
.endliteral

;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------
AREA InterruptRAM (RAM, REL, CON)

 
area    text(ROM,REL)

;-----------------------------------------------------------------------------
;  FUNCTION NAME: GlobalSelectConnect
;
;  DESCRIPTION:
;    Sets the appropriate bit of the PRTxGS register to connect the pin to the 
;    global bus.
;
;-----------------------------------------------------------------------------
;
;   ARGUMENTS: 
;     A => pin mask  (ie  0 => 0x01, 1 => 0x02, 2 => 0x04, etc)
;     X => Address of actual port.  
;    
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE: Sets Global Select register for pin
;
;------------------------------------------------------------------------------
; Stack offset constants
    GSpin:          equ  0
    GSport:         equ -1

_GlobalSelectConnect:
 GlobalSelectConnect:
    RAM_PROLOGUE RAM_USE_CLASS_2
    push  X                                   ; stores pin and port in temp stack                                
    mov   X,SP                                ; locations. X is index to locations
    add   SP,1                                ; complement A to set up mask
    mov   [X + GSpin],A
    mov   A,[X + GSport]
    swap  X,A
    mov   A,reg[X+GLOBAL_SELECT_OFFSET]       ; mov GS pin into A
    mov   X,SP
    dec   X 
    or    A,[X + GSpin]                       ; and with mask
    push  A
    mov   A,[X + GSport]
    swap  A,X
    pop   A
    mov   reg[X+GLOBAL_SELECT_OFFSET],A       ; write back to GS pin
    add   SP,-2
    RAM_EPILOGUE RAM_USE_CLASS_2
    ret
;-----------------------------------------------------------------------------
;  FUNCTION NAME: GlobalSelectDisconnect
;
;  DESCRIPTION:
;    Clears the appropriate bit of the PRTxGS register to disconnect the pin 
;    from the global bus.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: 
;    A => pin mask  (ie  0 => 0x01, 1 => 0x02, 2 => 0x04, etc)
;    X => Address of actual port.  
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE: Clears Global Select register for pin
;
;------------------------------------------------------------------------------
; Stack offset constants
    GSpin:          equ  0
    GSport:         equ -1 
_GlobalSelectDisconnect:
 GlobalSelectDisconnect:
    RAM_PROLOGUE RAM_USE_CLASS_2
                                             ;Clears PRTxGS bit in pin position
      push  X
      mov   X,SP
      cpl   A
      add   SP,1                             ; complement A to set up mask
      mov   [X + GSpin],A
      mov   A,[X + GSport]
      swap  X,A
      mov   A,reg[X+GLOBAL_SELECT_OFFSET]    ; mov GS pin into A
      mov   X,SP
      dec   X 
      and   A,[X + GSpin]                    ; and with mask
      push  A
      mov   A,[X + GSport]
      swap  A,X
      pop   A
      mov   reg[X+GLOBAL_SELECT_OFFSET],A    ; write back to GS pin
      add   SP,-2
    RAM_EPILOGUE RAM_USE_CLASS_2
    ret
  
;-----------------------------------------------------------------------------
;  FUNCTION NAME: DiscreteDriveMode
;
;  DESCRIPTION:
;    Sets the drive mode for the pin
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:  ( FASTCALL16 format)
;    [SP-3] => pin (ie  0 => 0x01, 1 => 0x02, 2 => 0x04, etc)
;    [SP-4] => Actual port address
;    [SP-5] => drive mode
; 
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE: 
;
;------------------------------------------------------------------------------
; Stack offset constants
    DMpin:          equ -3
    DMport:         equ -4                   
    DriveMode:      equ -5
    

_DiscreteDriveMode:
 DiscreteDriveMode:
    RAM_PROLOGUE RAM_USE_CLASS_2
    M8C_SetBank1                               ;Set PRTxDM0 bit  in pin position
    mov   X,SP                                                  
    mov   A,[X + DriveMode]                    ;move drivemode to A
    and   A,0x01
    jnz   SetOne                               ; jmp if bit 0 set to 1
    mov   A,[X + DMpin]                        ; load pin to A
    cpl   A                                    ; set up as mask
    mov   [X + DMpin],A                        ; move back
    mov   A,[X + DMport]                       ; load port
    swap  X,A                                  ; X now points to port, A contains SP
    push  A                                    ; save on stack
    mov   A,reg[X+DRIVE_MODE_ZERO_OFFSET]      ; load contents of PRTxDM0 into A
    pop   X                                    ; restore SP
    and   A,[X + DMpin]                        ; set bit to zero
    push  A                                    ; push result on stack
    mov   A,[X + DMport]                       ; move port to A
    swap  X,A                                  ; X now points to port
    pop   A                                    ; restore from stack	
    mov   reg[X+DRIVE_MODE_ZERO_OFFSET],A      ; move to PRTxDM0
    mov   X,SP                                 ; restore SP
    mov   A,[X + DMpin]                        ; restore pin to unmasked value
    cpl   A                            
    mov   [X + DMpin],A                 
    jmp   Done

SetOne:
    mov   A,[ X + DMport]                      ; load port to A
    swap  X, A                                 ; X now points to port, A contains SP
    push  A                                    ; save the SP	
    mov   A,reg[X+DRIVE_MODE_ZERO_OFFSET]      ; load contents of PRTxDM0 into A
    pop   X                                    ; restore SP                         
    or    A,[X + DMpin]                        ; set to 1
    push  A                                    ; save on stack
    mov   A,[X + DMport]                       ; load port to A                             
    swap  X,A                                  ; X now points to port
    pop   A                                    ; restore from stack
    mov   reg[X+DRIVE_MODE_ZERO_OFFSET],A      ; move to PRTxDM0
    mov   X,SP                                 ; restore SP

Done:
                                               ; Set PRTxDM1 bit
    mov   A,[X + DriveMode]                    ; load drive mode to A
    and   A, 0x02
    asr   A
    jnz   SetOne1                              ;jmp if bit 2 set to 1                             
    mov   A,[X + DMpin]                        ; load pin to A
    cpl   A                                    ; set up as mask
    mov   [X + DMpin], A                       ; move back
    mov   A,[X + DMport]                       ; load port
    swap  X,A                                  ; X now points to port, A contains SP
    push  A                                    ; save on stack
    mov   A,reg[X+DRIVE_MODE_ONE_OFFSET]       ; load contents of PRTxDM1 into A
    pop   X                                    ; restore SP
    and   A,[X + DMpin]                        ; set bit to zero
    push  A                                    ; push result on stack
    mov   A,[X + DMport]                       ; move port to A                             
    swap  X,A                                  ; X now points to port
    pop   A                                    ; restore from stack	
    mov   reg[X+DRIVE_MODE_ONE_OFFSET],A       ; Set PRTxDM1 bit to zero
    mov   X,SP                                 ; restore SP
    mov   A,[X + DMpin]                        ; restore pin to unmasked value
    cpl   A                              
    mov   [X + DMpin],A                   
    jmp   Done1

SetOne1:
    mov   A,[X + DMport]                        ; load port to A 
    swap  X,A                                   ; X now points to port, A contains SP
    push  A                                     ; save the SP	
    mov   A,reg[X+DRIVE_MODE_ONE_OFFSET]        ; load contents of PRTxDM1 into A
    pop   X                                     ; restore SP
    or    A,[X + DMpin]                         ; set to 1                         
    push  A                                     ; save on stack                             
    mov   A,[X + DMport]                        ; load port to A                                               
    swap  X,A                                   ; X now points to port                            
    pop   A                                     ; restore from stack
    mov   reg[X+DRIVE_MODE_ONE_OFFSET],A        ; move to PRTxDM1
    mov   X,SP                                  ; restore SP

Done1:
    M8C_SetBank0                                ; Set PRTxDM2 bit
    mov   A,[X + DriveMode]                     ; load drive mode to A
    and   A,0x04
    asr   A
    asr   A
    jnz SetOne2                                 ; jump to set bit to 1
    mov   A,[X + DMpin]                         ; load pin to A
    cpl   A                                     ; set up as mask
    mov   [X + DMpin],A                         ; move back
    mov   A,[X + DMport]                        ; load port
    swap  X,A                                   ; X now points to port, A contains SP
    push  A                                     ; save on stack
    mov   A,reg[X+DRIVE_MODE_TWO_OFFSET]        ; load contents of PRTxDM2 into A
    pop   X                                     ; restore SP
    and   A,[X + DMpin]                         ; set bit to zero
    push  A                                     ; push result on stack
    mov   A,[X + DMport]                        ; move port to A                             
    swap  X,A                                   ; X now points to port
    pop   A                                     ; restore from stack
    mov   reg[X+DRIVE_MODE_TWO_OFFSET],A        ; move to PRTxDM2
    mov   X,SP                                  ; restore SP
    mov   A,[X + DMpin]                         ; restore pin to unmasked value
    cpl   A                              
    mov   [X + DMpin],A 
    jmp   Done2

SetOne2:
    mov   A,[X + DMport]                        ; load port to A 
    swap  X,A                                   ; X now points to port, A contains SP
    push  A                                     ; save the SP		
    mov   A,reg[X+DRIVE_MODE_TWO_OFFSET]        ; load contents of PRTxDM2 into A
    pop   X                                     ; restore SP
    or    A,[X + DMpin]                         ; set to 1                         
    push  A                                     ; save on stack                             
    mov   A,[X + DMport]                        ; load port to A                                               
    swap  X,A                                   ; X now points to port                            
    pop   A                                     ; restore from stack
    mov   reg[X+DRIVE_MODE_TWO_OFFSET],A
    mov   X,SP                                  ; restore SP    

Done2:
    RAM_EPILOGUE RAM_USE_CLASS_2
    ret

;-----------------------------------------------------------------------------
;  FUNCTION NAME: DISetPin
;
;  DESCRIPTION:
;    This function is a wrapper for SetPin that disables IRQs
;    before and re-enables IRQs after function is called.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: pin passed in A, port passed in X by fastcall16
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: Data_SHADE RAM image of port written 
;
;  THEORY of OPERATION or PROCEDURE: Sets pin hi
;
;------------------------------------------------------------------------------
_DISetPin:
 DISetPin:
    M8C_DisableGInt                 ;  Disable Global Irqs
    call  SetPin
    M8C_EnableGInt                  ;  Enable Global Irqs
    ret

.LITERAL
PORT_MASKS:
    DB   0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80
.ENDLITERAL

;-----------------------------------------------------------------------------
;  FUNCTION NAME: SetPin()
;
;  DESCRIPTION:
;    This function drives a given pin to a high or low state.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: 
;     A  =>  Pin and Port info  pin = A[3:0],  port = A[7:4]
;     X  =>  1 => Drive pin high,  0 => Drive pin Low
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: Data_SHADE RAM image of port written 
;
;  THEORY of OPERATION or PROCEDURE: 
;
;
;  IMPORTANT:
;    It is assumed that port shadow registers are sequencial.
;------------------------------------------------------------------------------
; Stack offset constants
   SPPIN:          equ  0
   SPPORT:         equ  1 
   SPOUT:          equ -1 
_SetPin:
 SetPin:
   RAM_PROLOGUE RAM_USE_CLASS_4

   RAM_SETPAGE_IDX  >Port_0_Data_SHADE        ; Set the IDX page pointer to the SHADE Ram
   push  X                     ; Save Output
   mov   X,SP       
   add   SP,2                  ; Make room for pin and port info

   mov   [X + SPPIN],A         ; Save a copy for pin
   and   [X + SPPIN],0x0F      ; Mask off only the pin info
   asr   A
   asr   A
   asr   A
   asr   A
   and   A,0x0F                ; Mask off just port info
   mov   [X + SPPORT],A        ; Save port
     
   mov   A,[X + SPPIN]         ; Convert pin to mask
   index PORT_MASKS
   mov   [X + SPPIN],A

   tst   [X + SPOUT],0xFF      ; Check if the pin should be turned on or off
   jnz   SP_SET_PIN_HIGH

SP_SET_PIN_LOW:                ; Set pin low
   mov   A,[X + SPPIN]
   cpl   A                     ; Invert pin Mask
   mov   X,[X + SPPORT]        ; Put port number in X
   RAM_CHANGE_PAGE_MODE FLAG_PGMODE_10b ; Use IDX_PP for indexing
   and   [X + Port_0_Data_SHADE],A    ; Or in the bit to be set.        
   jmp   SetPin_DoIt

SP_SET_PIN_HIGH:               ; Set pin high
   mov   A,[X + SPPIN]
   mov   X,[X + SPPORT]        ; Put port number in X
   RAM_CHANGE_PAGE_MODE FLAG_PGMODE_10b ; Use IDX_PP for indexing
   or    [X + Port_0_Data_SHADE],A    ; Or in the bit to be set.        

SetPin_DoIt:
   mov   A,[X + Port_0_Data_SHADE]    ; Put Copy of port in A.
   RAM_CHANGE_PAGE_MODE FLAG_PGMODE_11b ; Back to native mode        
   swap  A,X
   asl   A
   asl   A
   swap  A,X
   mov   reg[X],A              ; Write new value to port

SetPin_End:
   add   SP,-3
   RAM_EPILOGUE RAM_USE_CLASS_4
   ret



;-----------------------------------------------------------------------------
;  FUNCTION NAME: DiscreteReadPin
;
;  DESCRIPTION:
;    Returns a 1 if the particular pin is high, a 0 if it is low.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: ( fastcall16 format )
;    A => pin mask  (ie  0 => 0x01, 1 => 0x02, 2 => 0x04, etc)
;    X => Address of actual port. 
;
;  RETURNS: 
;    State of pin
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE: 
;
;------------------------------------------------------------------------------
; Stack offset constants
    GSpin:          equ  0
    GSport:         equ -1
_DiscreteReadPin:
DiscreteReadPin:
    RAM_PROLOGUE RAM_USE_CLASS_2
    push  X                                     ; save data register offset on stack
    mov   X,SP
    add   SP,1
    mov   [X + GSpin],A                         ; save pin data on stack
    mov   A,[X + GSport]
    swap  X,A
    mov   A,reg[X+DATA_REGISTER_OFFSET]         ; restore X pointer to DR and load pin contents
    mov   X,SP                                  ; restore SP in X  
    dec   X
    and   A,[X + GSpin]                         ; and with pin contents
    jz    EndChk
    mov   A,01h                                 ; return 1 if set.
    EndChk:
    add   SP,-2
    RAM_EPILOGUE RAM_USE_CLASS_2 
    ret
; End of File DIO_LLD.asm
